from setuptools import setup, find_packages
setup(
    name='hyyToolbox',
    version='1.0.4',
    description='hyyToolbox have Administrator Privileges and Create a Shortcut...',
    author='huang yi yi',
    author_email='363766687@qq.com',
    packages=find_packages(),
    py_modules=['hyyToolbox','Windows','download','other']
)